#include "include/common.h"

int db_open( struct db_t *db, char *filename )
{
	extern struct config_t	config;
	
	struct stat		stat;
	int			db_res;
	char			filepath[1024*4];

	if ( db == NULL )
		return SQLITE_ERROR;

	snprintf( filepath, 1024*4, "%s/%s", config.db_path, filename );

	if ( lstat( filepath, &stat ) == 0 && ( S_ISREG(stat.st_mode) || S_ISLNK(stat.st_mode) ) )
	{
		db_res = sqlite3_open( filepath, &db->db );

		if ( db_res == SQLITE_OK )
		{
			db->opened = TRUE;
			return SQLITE_OK;
		}
		return db_res;
	}
	return SQLITE_NOTFOUND;
}

int db_open_init( struct db_t *db, char *filename, char *init_sql )
{
	extern struct config_t	config;
	
	struct stat		stat;
	int			db_res;
	short			need_init = TRUE;
	char			filepath[1024*4];

	if ( db == NULL )
		return SQLITE_ERROR;
		
	snprintf( filepath, 1024*4, "%s/%s", config.db_path, filename );

	if ( lstat( filepath, &stat ) == 0 && ( S_ISREG(stat.st_mode) || S_ISLNK(stat.st_mode) ) )
	{
		need_init = FALSE;
	}

	db_res = sqlite3_open( filepath, &db->db );
	if ( db_res == SQLITE_OK )
	{
		db->opened = TRUE;
		if ( need_init == TRUE && init_sql != NULL )
		{
			db_throw( db, init_sql );
			db_err_free( db );
		}
		return SQLITE_OK;
	}
	return db_res;
}

int db_close( struct db_t *db )
{
	int	result;

	if ( db->opened == TRUE )
	{
		db_err_free( db );
		result = sqlite3_close( db->db );
		db->opened = FALSE;
		db->db = NULL;
		return result;
	}
	return SQLITE_OK;
}

int db_set_authorizer( struct db_t *db, void *data )
{
	return sqlite3_set_authorizer( db->db, authorizer, data );
}

int db_throw( struct db_t *db, char *sql )
{
	if ( db == NULL )
		return SQLITE_ERROR;
	return sqlite3_exec( db->db, sql, NULL, NULL, &db->errmsg );
}

int db_commit( struct db_t *db )
{
	return db_throw( db, "COMMIT" );
}

int db_rollback( struct db_t *db )
{
	return db_throw( db, "ROLLBACK" );
}

char *db_printf( const char *sql, ...)
{
	va_list	argv;
	char	*dst;

	va_start( argv, sql );
	dst = sqlite3_vmprintf( sql, argv );
	va_end( argv );

	return dst;
}

void db_err_free( struct db_t *db )
{
	if ( db->errmsg != NULL )
		db_free( db->errmsg );
}

void db_free( char *sql )
{
	sqlite3_free( sql );
}

int db_changes( struct db_t *db )
{
	if ( db->db == NULL )
		return 0;
	return sqlite3_changes( db->db );
}

long long int db_last_rowid( struct db_t *db )
{
	if ( db->db == NULL )
		return 0;
	return sqlite3_last_insert_rowid( db->db );
}

int db_get_table( struct db_t *db, char *sql )
{
	if ( db == NULL )
		return SQLITE_ERROR;
	db->table_res = sqlite3_get_table( db->db, sql, &db->table, &db->table_nrow, &db->table_ncol, &db->errmsg );
	return db->table_res;
}

int db_table_columns( struct db_t *db )
{
	if ( db->table_res == SQLITE_OK )
		return db->table_ncol;
	return -1;
}

int db_table_column_name( char **column_name, struct db_t *db, int offset )
{
	if ( db->table_res == SQLITE_OK )
	{
		(*column_name) = db->table[offset];
		return strlen( (*column_name) );
	}
	else
	{
		(*column_name) = "";
		return 0;
	}
}

int db_table_num_rows( struct db_t *db )
{
	if ( db->table_res == SQLITE_OK )
		return db->table_nrow;
	return -1;
}

int db_table_has_rows( struct db_t *db )
{
	if ( db->table_res == SQLITE_OK )
		return (db->table_nrow > 0)?SQLITE_OK:SQLITE_ERROR;
	return SQLITE_ERROR;
}

char *db_table_result( struct db_t *db, int row, int col )
{
	if ( db->table_res == SQLITE_OK && db->table_nrow >= row )
		return db->table[(db->table_ncol * (row+1) + col)];
	return NULL;
}

void db_free_table( struct db_t *db )
{
	if ( db->table != NULL )
	{
		_db_free_table( db->table );
		db->table = NULL;
	}

	db->table_res = db->table_nrow = db->table_ncol = -1;
}

void _db_free_table( char **table )
{
	return sqlite3_free_table( table );
}

const char *db_version()
{
	return sqlite3_libversion();
}

int db_statement( struct db_t *db, char *sql )
{
	if ( db == NULL )
		return SQLITE_ERROR;
	return sqlite3_prepare( db->db, sql, strlen(sql), &(db->stmt), db->sql_tail );
}

int db_statement_column_count( struct db_t *db )
{
	if ( db == NULL || db->stmt == NULL )
		return SQLITE_ERROR;
	return sqlite3_column_count( db->stmt );
}

const char *db_statement_column_type( struct db_t *db, int seq )
{
	if ( db == NULL || db->stmt == NULL )
		return "";
	return sqlite3_column_decltype( db->stmt, seq );
}

const char *db_statement_column_name( struct db_t *db, int seq )
{
	if ( db == NULL || db->stmt == NULL )
		return "";
	return sqlite3_column_name( db->stmt, seq );
}

int db_statement_free( struct db_t *db )
{
	int	result;
	result=sqlite3_finalize( db->stmt );
	db->stmt = NULL;
	return result;
}

int db_column_meta( struct db_t *db, char *table, const char *column, struct desc_t *desc )
{
#ifdef SQLITE_ENABLE_COLUMN_METADATA
	if ( db == NULL )
		return SQLITE_ERROR;
	return sqlite3_table_column_metadata( db->db, NULL, table, column, &(desc->type), &(desc->seq), &(desc->notnull), &(desc->pkey), &(desc->autoincrement) );
#else
	return SQLITE_ERROR;
#endif
}

const char *db_error( struct db_t *db )
{
	if ( db == NULL )
		return "No database";
	return sqlite3_errmsg( db->db );
}
